global dic_hpList
dic_hpList = {
    "acptheme.cpskin3": [
        "ATVocabularyManager",
        "CacheSetup",
        "CMFSquidTool",
        "Clouseau",
        "CPUtils",
        "ImageEditor",
        "LinguaPlone",
        "Marshall",
        "NuPlone",
        "Ploneboard",
        "RICDescriptive",
        "SimpleAttachment",
        "SmartColorWidget",
        "ZopeRepository",
        "collective.js.jquery",
        "collective.js.jqueryui",
        "plone.app.iterate",
        "plone.app.openid",
        "plone.app.z3cform",
    ],
    "urban": ["produit 1", "produit 3", "produit 4"],
    "PloneMeeting:": [],
    "TeleServices": [],
}
